package org.rcisoft.business.memInfo.service.impl;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import jakarta.servlet.http.HttpServletRequest;
import org.apache.commons.lang3.StringUtils;
import org.rcisoft.alibaba.faceVerify.bean.FaceVerifyAliComp;
import org.rcisoft.alibaba.faceVerify.bean.FaceVerifyRedisBean;
import org.rcisoft.business.memInfo.bean.MemberGenerateBean;
import org.rcisoft.business.memInfo.bean.MemberInfoRedisBean;
import org.rcisoft.business.memInfo.entity.*;
import org.rcisoft.core.constant.CyDelStaCons;
import org.rcisoft.core.constant.CyFlagStaCons;
import org.rcisoft.core.exception.CyServiceException;
import org.rcisoft.core.service.CyRedisService;
import org.rcisoft.core.util.*;
import org.rcisoft.core.model.CyPersistModel;
import org.rcisoft.business.memInfo.dao.MemInfoRepository;
import org.rcisoft.business.memInfo.service.MemInfoService;

import org.rcisoft.sys.constant.CyUserCons;
import org.rcisoft.sys.dictionary.entity.DictData;
import org.rcisoft.sys.dictionary.service.DictionaryService;
import org.rcisoft.sys.rbac.user.dao.SysUserRbacRepository;
import org.rcisoft.sys.rbac.user.dto.SysUserRbacDTO;
import org.rcisoft.sys.rbac.user.entity.SysUserRbac;
import org.rcisoft.sys.rbac.user.enums.UserInfoExceptionEnums;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.security.crypto.password.PasswordEncoder;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Isolation;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;
import org.rcisoft.core.model.CyPageInfo;

import java.time.LocalDate;
import java.util.*;
import java.util.stream.Collectors;

import lombok.extern.slf4j.Slf4j;
import org.springframework.web.context.request.RequestContextHolder;
import org.springframework.web.context.request.ServletRequestAttributes;

/**
 * Created by cy on 2024年12月20日 下午3:35:34.
 */
@Service
@Transactional(readOnly = true,propagation = Propagation.NOT_SUPPORTED)
@Slf4j
public class MemInfoServiceImpl extends ServiceImpl<MemInfoRepository,MemInfo>  implements MemInfoService {

    // 定义允许的字符集
    private static final String CHAR_POOL = "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789_";
    private static final Random RANDOM = new Random();
    private static final Set<String> generatedNicknames = new HashSet<>();

    @Autowired
    private SysUserRbacRepository sysUserRbacRepository;
    @Autowired
    private PasswordEncoder passwordEncoder;
    @Autowired
    CyIdGenUtil cyIdGenUtil;
    @Autowired
    MemberGenerateBean memberGenerateBean;
    @Autowired
    private FaceVerifyAliComp faceVerifyAliComp;
    @Autowired
    private CyRedisService cyRedisServiceImpl;
    @Autowired
    private DictionaryService dictionaryService;
    @Autowired
    private CyRedisService cyRedisService;
    @Autowired
    private StringRedisTemplate redisTemplate;
    @Autowired
    private MemInfoRepository memInfoRepository;

    @Value("${cy.init.password}")
    private String password;


    /**
    * 保存 会员表
    * @param memInfo
    * @return
    */
    @Transactional(propagation = Propagation.REQUIRED,isolation = Isolation.DEFAULT)
    @Override
    public CyPersistModel persist(MemInfo memInfo){
        //手机号、身份证存储加密
        if (StringUtils.isNotBlank(memInfo.getMemPhone()))
            memInfo.setMemPhone(CyAESUtils.encrypt(memInfo.getMemPhone()));
        if (StringUtils.isNotBlank(memInfo.getMemIdcard()))
            memInfo.setMemIdcard(CyAESUtils.encrypt(memInfo.getMemIdcard()));
        if (StringUtils.isNotBlank(memInfo.getMemRealName()))
            memInfo.setMemRealName(CyAESUtils.encrypt(memInfo.getMemRealName()));
        //身份证、手机号验重
        SysUserRbacDTO sysUserRbacDTO = baseMapper.checkPhoneRepeat(null,memInfo.getMemPhone());
        if (sysUserRbacDTO != null) {
            throw new CyServiceException(UserInfoExceptionEnums.PHONE_EXISTS);
        }
        sysUserRbacDTO = baseMapper.checkIdcardRepeat(null,memInfo.getMemIdcard());
        if (sysUserRbacDTO != null) {
            throw new CyServiceException(UserInfoExceptionEnums.ID_NUMBER_EXISTS);
        }
        //添加sys_user
        SysUserRbac userRbac = new SysUserRbac();
        userRbac.setUsername(this.generateRandomName());
        userRbac.setUserType(CyUserCons.USER_TYPE_COMMON);
        userRbac.setPhone(memInfo.getMemPhone());
        userRbac.setSex(memInfo.getMemSex());
        userRbac.setPassword(passwordEncoder.encode(password));
        userRbac.setIdNumber(memInfo.getMemIdcard());
        userRbac.setUserIdentity("10");
        userRbac.setFlag(CyFlagStaCons.NORMAL.getStatus());
        userRbac.setDelFlag(CyDelStaCons.NORMAL.getStatus());
        userRbac.setWxOpenid(memInfo.getWxOpenid());
//        userRbac.setAvatar(memInfo.getAvatar());
        userRbac.setNickName(memInfo.getMemNickName());
        sysUserRbacRepository.insert(userRbac);
        //查询会员角色
        List<Integer> roleIds = baseMapper.selectRoleByKey("member");
        //插入角色
        if (!roleIds.isEmpty()) {
            sysUserRbacRepository.insertUserRole(userRbac.getBusinessId(), roleIds);
        }
        //生成会员号
        String maxCode = baseMapper.selectMaxCode();
        String memCode = generateMemCode(maxCode);
        memInfo.setMemCode(memCode);
        //增加操作
        memInfo.setUserId(userRbac.getBusinessId());
        memInfo.setSort(0);
        memInfo.setIsRecommended(0);
        int line = baseMapper.insert(memInfo);
        log.debug(CyUserUtil.getAuthenUsername()+"新增了ID为"+
            memInfo.getBusinessId()+"的会员表信息");
        return new CyPersistModel(line);
    }

    /**
    * 删除 会员表
    * @param memInfo
    * @return
    */
    @Transactional(propagation = Propagation.REQUIRED,isolation = Isolation.DEFAULT)
    @Override
    public CyPersistModel remove(MemInfo memInfo){
        int line = baseMapper.realDelete(memInfo);
        log.debug(CyUserUtil.getAuthenUsername()+"删除了ID为"+
            memInfo.getBusinessId()+"的会员表信息");
        return new CyPersistModel(line);
    }

    /**
    * 逻辑删除 会员表
    * @param memInfo
    * @return
    */
    @Transactional(propagation = Propagation.REQUIRED,isolation = Isolation.DEFAULT)
    @Override
    public CyPersistModel removeLogical(MemInfo memInfo){
        memInfo.setDeleted();
        int line = baseMapper.deleteById(memInfo);
            log.debug(CyUserUtil.getAuthenUsername()+"逻辑删除了ID为"+
                memInfo.getBusinessId()+"的会员表信息");
            return new CyPersistModel(line);
    }

    /**
    * 修改 会员表
    * @param memInfo
    * @return
    */
    @Transactional(propagation = Propagation.REQUIRED,isolation = Isolation.DEFAULT)
    @Override
    public CyPersistModel merge(MemInfo memInfo){
        //手机号、身份证存储加密
        if (StringUtils.isNotBlank(memInfo.getMemPhone()))
            memInfo.setMemPhone(CyAESUtils.encrypt(memInfo.getMemPhone()));
        if (StringUtils.isNotBlank(memInfo.getMemIdcard()))
            memInfo.setMemIdcard(CyAESUtils.encrypt(memInfo.getMemIdcard()));
        if (StringUtils.isNotBlank(memInfo.getMemRealName()))
            memInfo.setMemRealName(CyAESUtils.encrypt(memInfo.getMemRealName()));
        //身份证、手机号验重
        SysUserRbacDTO sysUserRbacDTO = baseMapper.checkPhoneRepeat(memInfo.getUserId(),memInfo.getMemPhone());
        if (sysUserRbacDTO != null) {
            throw new CyServiceException(UserInfoExceptionEnums.PHONE_EXISTS);
        }
        sysUserRbacDTO = baseMapper.checkIdcardRepeat(memInfo.getUserId(),memInfo.getMemIdcard());
        if (sysUserRbacDTO != null) {
            throw new CyServiceException(UserInfoExceptionEnums.ID_NUMBER_EXISTS);
        }
        //修改sys_user
        SysUserRbac userRbac = new SysUserRbac();
        userRbac.setBusinessId(memInfo.getUserId());
        userRbac.setPhone(memInfo.getMemPhone());
        userRbac.setSex(memInfo.getMemSex());
        userRbac.setIdNumber(memInfo.getMemIdcard());
//        userRbac.setAvatar(memInfo.getAvatar());
        userRbac.setNickName(memInfo.getMemNickName());
        sysUserRbacRepository.updateById(userRbac);
        int line = baseMapper.updateById(memInfo);
        log.debug(CyUserUtil.getAuthenUsername()+"修改了ID为"+ memInfo.getBusinessId()+"的会员表信息");
        return new CyPersistModel(line);
    }

    /**
    * 根据id查询 会员表
    * @param id
    * @return
    */
    @Override
    public MemInfo findById(int id){
        //查询基本信息
        MemInfo memInfo = baseMapper.selectDetailById(id);
        //查询我喜欢的 喜欢我的  互相喜欢的数量
        MemInfo memInfo1 =  baseMapper.selectUserFollowCount(memInfo.getUserId());
        memInfo.setFollowMeCount(memInfo1.getFollowMeCount());
        memInfo.setMeFollowCount(memInfo1.getMeFollowCount());
        memInfo.setEachFollowCount(memInfo1.getEachFollowCount());
        return memInfo;
    }

    /**
    * 分页查询 会员表
    * @param memInfo
    * @return
    */
    @Override
    public IPage<MemInfo> findAllByPagination(CyPageInfo<MemInfo> paginationUtility,
        MemInfo memInfo){
        if (memInfo.getMinAge()!=null && memInfo.getMaxAge()!=null) {
            memInfo.setBeginBirthday(computeBirthdayBegin(memInfo.getMaxAge()));
            memInfo.setEndBirthday(computeBirthdayEnd(memInfo.getMinAge()));
        }
        //去除小程序端首页的登录人显示  不能自己热门推荐自己
        String loginUserId = CyUserUtil.getAuthenBusinessId();
        if (StringUtils.isNotEmpty(loginUserId)){
            memInfo.setLoginUserId(Integer.valueOf(loginUserId));
        }
        //查询我喜欢的 喜欢我的  互相喜欢的会员列表（应用于小程序端-我的）
        if (StringUtils.isNotEmpty(memInfo.getQueryType())){
            if (memInfo.getQueryType().equals("1")){
                //我喜欢的
                return baseMapper.queryMeFollowPaged(paginationUtility,memInfo);
            } else if(memInfo.getQueryType().equals("2")){
                //喜欢我的
                return baseMapper.queryFollowMePaged(paginationUtility,memInfo);
            } else if(memInfo.getQueryType().equals("3")){
                //互相喜欢的
                return baseMapper.queryEachFollowPaged(paginationUtility,memInfo);
            }
        }
        //查询会员表集合
        return baseMapper.queryMemInfosPaged(paginationUtility,memInfo);

    }

    /**
     * 分页查询 会员表
     *
     * @param paginationUtility
     * @param memInfo
     * @return
     */
    @Override
    public IPage<MemInfo> findLikeListByPagination(CyPageInfo<MemInfo> paginationUtility, MemInfo memInfo) {
        String loginUserId = CyUserUtil.getAuthenBusinessId();
        if (StringUtils.isNotEmpty(loginUserId)){
            memInfo.setLoginUserId(Integer.valueOf(loginUserId));
        }
        //查询我喜欢的 喜欢我的  互相喜欢的会员列表（应用于小程序端-我的）
        if (StringUtils.isNotEmpty(memInfo.getQueryType())){
            if (memInfo.getQueryType().equals("1")){
                //我喜欢的
                return baseMapper.queryMeFollowPaged(paginationUtility,memInfo);
            } else if(memInfo.getQueryType().equals("2")){
                //喜欢我的
                return baseMapper.queryFollowMePaged(paginationUtility,memInfo);
            } else if(memInfo.getQueryType().equals("3")){
                //互相喜欢的
                return baseMapper.queryEachFollowPaged(paginationUtility,memInfo);
            }
        }
        return paginationUtility;
    }

    /**
    * 查询list 会员表
    * @param memInfo
    * @return
    */
    @Override
    public List<MemInfo> findAll(MemInfo memInfo){
        return baseMapper.queryMemInfos(memInfo);

    }

    /**
    * 导出会员表
    * @return
    */
    @Override
    public List<MemInfo> export(MemInfo memInfo) {
        List<MemInfo> memInfoList = baseMapper.queryMemInfos(memInfo);
        return memInfoList;
    }


    private String generateRandomName() {
        StringBuilder nickname;
        int length = 10;
        do {
            nickname = new StringBuilder(length);
            for (int i = 0; i < length; i++) {
                int index = RANDOM.nextInt(CHAR_POOL.length());
                nickname.append(CHAR_POOL.charAt(index));
            }
        } while (generatedNicknames.contains(nickname.toString()));
        // 存储生成的昵称
        generatedNicknames.add(nickname.toString());
        return "wx_" + nickname.toString();
    }


    /**
     * 校验会员信息
     * @param memInfo
     * @return
     */
    @Override
    public Boolean checkInfo(MemInfo memInfo) {
        //手机号、身份证存储加密
        if (StringUtils.isNotBlank(memInfo.getMemPhone()))
            memInfo.setMemPhone(CyAESUtils.encrypt(memInfo.getMemPhone()));
        if (StringUtils.isNotBlank(memInfo.getMemIdcard()))
            memInfo.setMemIdcard(CyAESUtils.encrypt(memInfo.getMemIdcard()));
        //身份证、手机号验重
        SysUserRbacDTO sysUserRbacDTO = baseMapper.checkPhoneRepeat(memInfo.getUserId(),memInfo.getMemPhone());
        if (sysUserRbacDTO != null) {
            throw new CyServiceException(UserInfoExceptionEnums.PHONE_EXISTS);
        }
        sysUserRbacDTO = baseMapper.checkIdcardRepeat(null,memInfo.getMemIdcard());
        if (sysUserRbacDTO != null) {
            throw new CyServiceException(UserInfoExceptionEnums.ID_NUMBER_EXISTS);
        }
        return true;
    }

    /**
     * 通过年龄计算出生日期开始
     * @param maxAge
     * @return
     */
    private static String computeBirthdayBegin(Integer maxAge) {
        // 获取当前日期
        LocalDate currentDate = LocalDate.now();
        // 计算出生年份：当前年份减去年龄-1
        int birthYear = currentDate.getYear() - maxAge - 1;
        // 计算出生日期：当前日期晚一天
        LocalDate birthDate = currentDate.plusDays(1);
        birthDate = birthDate.withYear(birthYear);
        return birthDate.toString();
    }


    /**
     * 通过年龄计算出生日期结束
     * @param minAge
     * @return
     */
    private static String computeBirthdayEnd(Integer minAge) {
        // 获取当前日期
        LocalDate currentDate = LocalDate.now();
        // 计算出生年份：当前年份减去年龄-1
        int birthYear = currentDate.getYear() - minAge;
        // 计算出生日期：当前日期晚一天
        LocalDate birthDate = currentDate.withYear(birthYear);
        return birthDate.toString();
    }

    /**
     * 生成会员号
     * @param code
     * @return
     */
    public String generateMemCode(String code) {  //将会员号段排序
        List<String> codeList = Arrays.stream(memberGenerateBean.getCodes()).sorted().collect(Collectors.toList());
        if (StringUtils.isNotBlank(code)) {
            int currentNumber = Integer.parseInt(code);
            //获取当前会员号所处的号段
            for (int i = 0; i < codeList.size(); i++) {
                String[] range = codeList.get(i).split("-");
                int minCode = Integer.parseInt(range[0]);
                int maxCode = Integer.parseInt(range[1]);
                if (currentNumber >= minCode && currentNumber <= maxCode) {
                    // 如果当前号码在号段内，返回该号段中的下一个号码
                    int nextNumber = currentNumber + 1;
                    if (nextNumber <= maxCode) {
                        // 获取当前号码的长度
                        int length = code.length();
                        // 格式化为4位数（根据需要调整位数）
                        return String.format("%0" + length + "d", nextNumber);
                    } else {
                        // 当前号码已经是该号段的最大值，返回下一个号段的起始值
                        if ((i + 1) < codeList.size())
                            return codeList.get(i + 1).split("-")[0];
                        throw new CyServiceException("会员号超出设定最大值，请联系管理员");
                    }
                }
            }
            return null;
        } else {
            return codeList.get(0).split("-")[0];
        }
    }



    @Override
    public MemInfo getUserPhone(Integer businessId) {
        MemInfo memInfo = baseMapper.getUserPhone(businessId);
        //存储加密信息解密
        if (StringUtils.isNotBlank(memInfo.getMemPhone()))
            memInfo.setMemPhone(CyAESUtils.decrypt(memInfo.getMemPhone()));
        if (StringUtils.isNotBlank(memInfo.getMemIdcard()))
            memInfo.setMemIdcard(CyAESUtils.decrypt(memInfo.getMemIdcard()));
        if (StringUtils.isNotBlank(memInfo.getMemRealName()))
            memInfo.setMemRealName(CyAESUtils.decrypt(memInfo.getMemRealName()));
        return memInfo;
    }


    @Override
    public IPage<opmArticleDTO> findArticleByPagination(CyPageInfo<MemInfo> paginationUtility,
                                                        opmArticleDTO opmArticleDTO){
        return baseMapper.queryOpmArticlesPaged(paginationUtility,opmArticleDTO);

    }

    @Override
    public peopleNumberDTO getPeopleNumber(peopleNumberDTO peopleNumberDTO ){
        peopleNumberDTO list = new peopleNumberDTO();

        list.setArticleNumber(baseMapper.getArticleNumber(peopleNumberDTO));
        list.setActivityNumber(baseMapper.getActivityNumber(peopleNumberDTO));
        list.setRealNumber(baseMapper.getRealNumber(peopleNumberDTO));
        list.setRegisterNumber(baseMapper.getRegisterNumber(peopleNumberDTO));
        list.setCmsActivityList(baseMapper.getCmsActivityList(peopleNumberDTO));
        list.setVisitCount(baseMapper.getCmsActivityList(peopleNumberDTO));

        return list;
    }


    @Override
    public ArticleExamStatus getArticle(ArticleExamStatus articleExamStatus ){

        LocalDate startDate = articleExamStatus.getDate().minusDays(7);
        LocalDate startDateMonth = articleExamStatus.getDate().minusMonths(7);
        LocalDate endDate = articleExamStatus.getDate();

        ArticleExamStatus list = new ArticleExamStatus();

        //审核通过
        list.setArticleExamStatusDayYes(baseMapper.getArticleCountByDateRangeYes(startDate, endDate));
        list.setArticleExamStatusMonthYes(baseMapper.getArticleCountByMonthRangeYes(startDateMonth, endDate));

        //审核未通过
        list.setArticleExamStatusDayNo(baseMapper.getArticleCountByDateRangeNo(startDate, endDate));
        list.setArticleExamStatusMonthNo(baseMapper.getArticleCountByMonthRangeNo(startDateMonth, endDate));

        return list;
    }


    /**
     * 修改 会员表
     * @param memInfo
     * @return
     */
    @Transactional(propagation = Propagation.REQUIRED,isolation = Isolation.DEFAULT)
    @Override
    public CyPersistModel checkIdCard(MemInfo memInfo){
        MemInfo info = baseMapper.getInfoByUserId(String.valueOf(memInfo.getUserId()));
        if ("1".equals(info.getMemRealAuthen()))
            throw new CyServiceException("用户已实名，无需再次实名认证");
        //手机号、身份证存储加密
//        if (StringUtils.isNotBlank(memInfo.getMemIdcard()))
//            info.setMemIdcard(CyAESUtils.encrypt(memInfo.getMemIdcard()));
//        if (StringUtils.isNotBlank(memInfo.getMemRealName()))
//            info.setMemRealName(CyAESUtils.encrypt(memInfo.getMemRealName()));
        //身份证、手机号验重
        SysUserRbacDTO sysUserRbacDTO = baseMapper.checkIdcardRepeat(memInfo.getUserId(),CyAESUtils.encrypt(memInfo.getMemIdcard()));
        if (sysUserRbacDTO != null) {
            throw new CyServiceException(UserInfoExceptionEnums.ID_NUMBER_EXISTS);
        }
        //验证失败次数
        //获取ip
        HttpServletRequest request = ((ServletRequestAttributes) RequestContextHolder.getRequestAttributes()).getRequest();
        String ip = CyAddressUtils.IpUtils.getIpAddr(request);
        //1. 同一IP，每分钟失败次数
        Object captcha = cyRedisServiceImpl.get(FaceVerifyRedisBean.FACEVERIFY_IP_MINUTES + ip);
        if (captcha != null) {
            int count = (int)captcha;
            if (count >= faceVerifyAliComp.getIpMinCount())
                throw new CyServiceException("实名认证发起频繁，请稍后重试");
        }
        //2. 同一IP，每天失败次数
        captcha = cyRedisServiceImpl.get(FaceVerifyRedisBean.FACEVERIFY_IP_DAY + ip);
        if (captcha != null) {
            int count = (int)captcha;
            if (count >= faceVerifyAliComp.getIpDayCount())
                throw new CyServiceException("实名认证发起频繁，请稍后重试");
        }
        //3. 同一身份证，每分钟失败次数
        captcha = cyRedisServiceImpl.get(FaceVerifyRedisBean.FACEVERIFY_IDCARD_MINUTES + memInfo.getMemIdcard());
        if (captcha != null) {
            int count = (int)captcha;
            if (count >= faceVerifyAliComp.getIdCardMinCount())
                throw new CyServiceException("实名认证发起频繁，请稍后重试");
        }
        //4. 同一身份证，每天失败次数
        captcha = cyRedisServiceImpl.get(FaceVerifyRedisBean.FACEVERIFY_IDCARD_DAY + memInfo.getMemIdcard());
        if (captcha != null) {
            int count = (int)captcha;
            if (count >= faceVerifyAliComp.getIdCardDayCount())
                throw new CyServiceException("实名认证发起频繁，请稍后重试");
        }
        cyRedisServiceImpl.set(FaceVerifyRedisBean.FACEVERIFY_USER+memInfo.getUserId(),memInfo,3600L);
        return new CyPersistModel(1);
    }

    /**
     * 修改 会员表
     * @param memInfo
     * @return
     */
    @Transactional(propagation = Propagation.REQUIRED,isolation = Isolation.DEFAULT)
    @Override
    public CyPersistModel update(MemInfo memInfo){
        MemInfo info = baseMapper.selectByOpenId(memInfo.getWxOpenid());
        //手机号、身份证存储加密
        if (StringUtils.isNotBlank(memInfo.getMemPhone()))
            memInfo.setMemPhone(CyAESUtils.encrypt(memInfo.getMemPhone()));
        memInfo.setMemIdcard(info.getMemIdcard());
        memInfo.setMemRealName(info.getMemRealName());
        memInfo.setBusinessId(info.getBusinessId());
        //身份证、手机号验重
        SysUserRbacDTO sysUserRbacDTO = baseMapper.checkPhoneRepeat(info.getUserId(),memInfo.getMemPhone());
        if (sysUserRbacDTO != null) {
            throw new CyServiceException(UserInfoExceptionEnums.PHONE_EXISTS);
        }
        //修改sys_user
        SysUserRbac userRbac = new SysUserRbac();
        userRbac.setBusinessId(info.getUserId());
        userRbac.setPhone(memInfo.getMemPhone());
        userRbac.setSex(memInfo.getMemSex());
        userRbac.setNickName(memInfo.getMemNickName());
        sysUserRbacRepository.updateById(userRbac);
        int line = baseMapper.updateById(memInfo);
        log.debug(CyUserUtil.getAuthenUsername()+"修改了ID为"+ memInfo.getBusinessId()+"的会员表信息");
        return new CyPersistModel(line);
    }
    /**
     * 添加关注
     *
     * @param followDTO
     * @return
     */
    @Transactional(propagation = Propagation.REQUIRED,isolation = Isolation.DEFAULT)
    @Override
    public CyPersistModel addFollow(MemFollowDTO followDTO){
        int line = baseMapper.addFollow(followDTO);
        return new CyPersistModel(line);
    }
    /**
     * 查询是否关注
     *
     * @param followDTO
     * @return
     */
    @Transactional(propagation = Propagation.REQUIRED,isolation = Isolation.DEFAULT)
    @Override
    public MemFollowDTO isFollow(MemFollowDTO followDTO){
        return baseMapper.getIsFollow(followDTO);
    }
    /**
     * 取消关注
     *
     * @param followDTO
     * @return
     */
    @Transactional(propagation = Propagation.REQUIRED,isolation = Isolation.DEFAULT)
    @Override
    public CyPersistModel deleteFollow(MemFollowDTO followDTO){
        Integer line = baseMapper.deleteFollow(followDTO);
        return new CyPersistModel(line);
    }

    /**
     * 用户点赞
     */
    @Override
    public CyPersistModel addLike(MemLikeDTO likeDTO) {
        Integer userId = Integer.valueOf(CyUserUtil.getAuthenBusinessId());
        //点赞配置 分为两种
        List<DictData> userLikeConfig = dictionaryService.selectByTypes("user_like_config");
        //一天最多给所有用户点赞的总次数
        Integer dicCountToAll = Integer.valueOf(userLikeConfig.get(0).getDictValue());
        //一天最多给某个人点赞的总次数
        Integer dicCountToPersonal = Integer.valueOf(userLikeConfig.get(1).getDictValue());
        //redis中 该用户对所有用户的点赞次数
        Object redisCountToAll = cyRedisServiceImpl.get(MemberInfoRedisBean.USER_LIKE_USERLIKEALL + userId);
        //redis中 该用户对目标用户的点赞次数
        Object redisCountToPersonal = cyRedisServiceImpl.hget(MemberInfoRedisBean.USER_LIKE_USERLIKE + userId, String.valueOf(likeDTO.getTargetId()));
        if (redisCountToAll == null && redisCountToPersonal == null){
            //用户第一次进行点赞 设置redis缓存
            //对所有用户的点赞次数存储
            cyRedisServiceImpl.set(MemberInfoRedisBean.USER_LIKE_USERLIKEALL + userId,1);
            cyRedisServiceImpl.hset(MemberInfoRedisBean.USER_LIKE_USERLIKE + userId, String.valueOf(likeDTO.getTargetId()),1);
            likeDTO.setUserId(userId);
            int line = baseMapper.addLike(likeDTO);
            return new CyPersistModel(line);
        } else if (redisCountToPersonal == null){
            //用户对目标用户是第一次点赞
            //判断点赞总数限制
            if (((int)redisCountToAll < dicCountToAll)){
                //条件限制都满足 点赞次数都+1
                cyRedisServiceImpl.incr(MemberInfoRedisBean.USER_LIKE_USERLIKEALL + userId, 1L);
                cyRedisServiceImpl.hincr(MemberInfoRedisBean.USER_LIKE_USERLIKE + userId, String.valueOf(likeDTO.getTargetId()), (double) 1L);
                likeDTO.setUserId(userId);
                int line = baseMapper.addLike(likeDTO);
                return new CyPersistModel(line);
            } else {
                throw new CyServiceException("点赞失败，今日点赞次数已达上限");
            }
        } else if (redisCountToPersonal != null){
            //判断点赞总数限制
            if (((int)redisCountToAll < dicCountToAll)){
                //判断个人点赞次数限制
                if ((int)redisCountToPersonal < dicCountToPersonal){
                    //条件限制都满足 点赞次数都+1
                    cyRedisServiceImpl.incr(MemberInfoRedisBean.USER_LIKE_USERLIKEALL + userId, 1L);
                    cyRedisServiceImpl.hincr(MemberInfoRedisBean.USER_LIKE_USERLIKE + userId, String.valueOf(likeDTO.getTargetId()), (double) 1L);
                } else {
                    throw new CyServiceException("点赞失败，今日对该用户的点赞次数已达上限");
                }
            } else {
                throw new CyServiceException("点赞失败，今日点赞次数已达上限");
            }
        }
        return new CyPersistModel(1);
    }

    /**
     * 查询排行榜
     * @param memInfo
     * @return
     */
    @Override
    public IPage<MemInfo> findRankByPagination(CyPageInfo<MemInfo> paginationUtility, MemInfo memInfo){
        return baseMapper.queryRankPaged(paginationUtility,memInfo);
    }


    /**
     * 用户点赞 定时任务
     */
    public void userLikeSchedule() {
        //将redis中存储的点赞数同步到opm_user_like表中
        Set<String> keys = redisTemplate.keys(MemberInfoRedisBean.USER_LIKE_USERLIKE + "*");
        if (keys != null && !keys.isEmpty()) {
            for (String key : keys) {
                //截取key最后一个:后的值，为userId
                int lastIndex = key.lastIndexOf(':');
                Integer userId = Integer.valueOf(key.substring(lastIndex + 1));
                //获取key的所有value
                Map<Object, Object> hmget = cyRedisService.hmget(key);
                if (hmget != null && !hmget.isEmpty()) {
                    //遍历map
                    for (Map.Entry<Object, Object> entry : hmget.entrySet()) {
                        //获取目标
                        String targetId = (String) entry.getKey();
                        //获取点赞数
                        Integer likeCount = (Integer) entry.getValue();
                        //将redis中存储的点赞数同步到opm_user_like表中
                        MemLikeDTO memLikeDTO = new MemLikeDTO();
                        memLikeDTO.setUserId(userId);
                        memLikeDTO.setLikeCount(likeCount);
                        memLikeDTO.setTargetId(Integer.valueOf(targetId));
                        //对点赞表 进行增加点赞数
                        memInfoRepository.addLikeCount(memLikeDTO);
                        //对会员表 进行增加被点赞数
                        memInfoRepository.addMemLikeCount(memLikeDTO);
                    }
                }
            }
        } else {
            System.out.println("No keys found.");
        }
        log.info("用户点赞数据定时任务执行中...");
    }


    /**
     * 留言
     * @param dto
     * @return
     */
    @Override
    public CyPersistModel leaveMessage(UserLeaveMessageDTO dto) {
        dto.setDelStatus("0");
        dto.setUserId(Integer.valueOf(CyUserUtil.getAuthenBusinessId()));
        int line = memInfoRepository.leaveMessage(dto);
        return new CyPersistModel(line);
    }

    /**
     * 分页查询留言列表
     *
     * @param paginationUtility
     * @param dto
     */
    @Override
    public IPage<MemInfo> leaveMessageByPagination(CyPageInfo<MemInfo> paginationUtility, UserLeaveMessageDTO dto) {
        dto.setUserId(Integer.valueOf(CyUserUtil.getAuthenBusinessId()));
        IPage<MemInfo> userLeaveMessageIPage = memInfoRepository.leaveMessageByPagination(paginationUtility, dto);
        return userLeaveMessageIPage;
    }

    /**
     * 查询留言列表详情 1对1的
     * @param paginationUtility
     * @param dto
     */
    @Override
    public IPage<MemInfo> leaveMessageDetailByPagination(CyPageInfo<MemInfo> paginationUtility, UserLeaveMessageDTO dto) {
        dto.setUserId(Integer.valueOf(CyUserUtil.getAuthenBusinessId()));
        IPage<MemInfo> userLeaveMessageIPage = memInfoRepository.leaveMessageDetailList(paginationUtility, dto);
        return userLeaveMessageIPage;
    }
}
