package org.rcisoft.core.schedule;

import lombok.extern.slf4j.Slf4j;
import org.jdom.JDOMException;
import org.rcisoft.business.cmsOrder.service.impl.CmsOrderServiceImpl;
import org.rcisoft.business.memInfo.bean.MemberInfoRedisBean;
import org.rcisoft.business.memInfo.dao.MemInfoRepository;
import org.rcisoft.business.memInfo.entity.MemLikeDTO;
import org.rcisoft.business.memInfo.service.MemInfoService;
import org.rcisoft.business.memInfo.service.impl.MemInfoServiceImpl;
import org.rcisoft.core.service.CyRedisService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.scheduling.annotation.EnableScheduling;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;

import java.io.IOException;
import java.security.NoSuchAlgorithmException;
import java.util.Map;
import java.util.Set;

@Component
@Slf4j
@EnableScheduling
public class ScheduleTasks {

   @Autowired
   private MemInfoServiceImpl memInfoService;
   @Autowired
   private CmsOrderServiceImpl orderService;

    /**
     * 用户点赞数据定时任务 将点赞数据同步到对应的表中
     * 会员表 点赞表
     */
    @Scheduled(cron = "0 0 0 * * ?")
    public void userLikeSchedule() {
        memInfoService.userLikeSchedule();
    }



    /**
     * redis订单落库
     */
    @Scheduled(cron = "0 0/1 * * * ?")
    public void executeInsertOrder() throws NoSuchAlgorithmException {
        orderService.executeInsertOrder();
    }

    /**
     * redis订单关闭
     */
    @Scheduled(cron = "0 0/1 * * * ?")
    public void executeCloseOrder() throws NoSuchAlgorithmException {
        orderService.executeCloseOrder();
    }


    /**
     * 支付确认补救
     */
    @Scheduled(cron = "0 0/15 * * * ?")
    public void executePayOrder() throws IOException, JDOMException {
        orderService.executePayOrder();
    }

    /**
     * 退款接口调用补救
     */
    @Scheduled(cron = "0 0/30 * * * ?")
    public void executeRefundOrder() {
        orderService.executeRefundOrder();
    }

    /**
     * 退款确认补救
     */
    @Scheduled(cron = "0 0/30 * * * ?")
    public void executeRefundConfirmOrder() throws IOException, JDOMException {
        orderService.executeRefundConfirmOrder();
    }

    /**
     * 订单缓存清除
     */
    @Scheduled(cron = "0 0 1 * * ?")
    public void executeDeleteOrder() {
        orderService.executeDeleteOrder();
    }



}
